// MySql1.cpp: Implementierung der Klasse CMySql.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdafx.h"
#include "MySql1.h"
#include "CPing.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CMySql::CMySql(CString host, CString user, CString password, CString database,
			   unsigned int port, BOOL compression, unsigned int timeout)
{
	m_host = host;
	m_user = user;
	m_password = password;
	m_database = database;
	m_port = port;
	m_compression = compression;
	m_timeout = timeout;
	m_lasterror.Empty();
	mysql_init( &m_mysql );

	m_result = NULL;
	m_ping_success = false;

	int ret = mysql_options( &m_mysql, MYSQL_OPT_CONNECT_TIMEOUT, &m_timeout);
	if (m_compression)
		mysql_options( &m_mysql, MYSQL_OPT_COMPRESS, &m_timeout);
	//ASSERT ( ret == 0);

	ConnectToHost();
}

CMySql::~CMySql()
{
	if (m_ping_success)
		mysql_close(&m_mysql);
}

BOOL CMySql::ConnectToHost()
{
	CPing ping;

	// check if host is online to avoid a lengthy pause
	if (!ping.Ping(m_host))
	{
		m_lasterror = "Host not reachable";
		return FALSE;
	}
	m_ping_success = true;
	if (mysql_real_connect( &m_mysql, (LPCTSTR)m_host, (LPCSTR)m_user,
						    (LPCTSTR)m_password, (LPCTSTR)m_database,
						    m_port, NULL, 0) == NULL)
	{
		GetLastError();
		return FALSE;
	}
	else
		return TRUE;
}

void CMySql::GetLastError()
{
	m_lasterror = mysql_error(&m_mysql);
}

BOOL CMySql::IsServerAlive()
{
	return (mysql_ping( &m_mysql ) == 0);
}

BOOL CMySql::Query(CString querystring)
{
	if (!IsServerAlive() && !ConnectToHost())
		return FALSE;

	if (m_result != NULL)
	{
		mysql_free_result(m_result);
		m_result = NULL;
	}
	if ( mysql_real_query(&m_mysql, (LPCTSTR)querystring, querystring.GetLength()) == 0 )
		return TRUE;
	else
	{
		GetLastError();
		return FALSE;
	}
}

BOOL CMySql::GetNextRow(MYSQL_ROW &row, BOOL &isValid)
{
	isValid = TRUE;
	if (m_result == NULL)
		if ( (m_result = mysql_store_result(&m_mysql)) == NULL )
		{
			GetLastError();
			return FALSE;
		}
	row = mysql_fetch_row( m_result );
	if (row == NULL)
	{
		isValid = FALSE;
		mysql_free_result( m_result );
		m_result = NULL;
	}
	return TRUE;
}

BOOL CMySql::GetResultStats(unsigned int &cols, unsigned int &rows)
{
	if (m_result == NULL)
		return FALSE;
	cols = mysql_num_fields( m_result );
	rows = (unsigned int)mysql_num_rows( m_result );
	return TRUE;
}

void CMySql::LastError(CString &errormsg)
{
	errormsg = m_lasterror;
}
